-- tanks.love

gr, li, ky, tm	= love.graphics, love.image, love.keyboard, love.timer
pi,sin,cos,atan2,floor = math.pi,math.sin,math.cos,math.atan2,math.floor

function love.load()
	load_screen_settings()
	load_defaults()
	end

function love.draw()
	gr.setColor( 255, 255, 255, 255 )
	circling_tanks()	-- tanks moving in ellipse
	rotating_tank()		-- center rotating tank
	path_of_tank()		-- calculate path for tank
	tank_on_path()		-- tank following path
	text_display()		-- display all enabled text
	end

function love.update(dt)
	tank_tread = tank_tread - 8 * dt		-- controls tank tread
	if floor( tank_tread ) > 1 then
		qdi = floor( tank_tread )
		else
		tank_tread, qdi = num_tanks, num_tanks
		end
	tank_step	= tank_step + 8 * dt
	if rotate 	  then r = r + dt/10 else r = 0 end
	for i		= 1, num_tanks do x[i], y[i] = path( i*dr + r ) end
	if ky.isDown('up')		then control_points[cp][2]=control_points[cp][2]-1 tank_path,curve=smooth(control_points,res)	end
	if ky.isDown('down')	then control_points[cp][2]=control_points[cp][2]+1 tank_path,curve=smooth(control_points,res)	end
	if ky.isDown('left')	then control_points[cp][1]=control_points[cp][1]-1 tank_path,curve=smooth(control_points,res)	end
	if ky.isDown('right')	then control_points[cp][1]=control_points[cp][1]+1 tank_path,curve=smooth(control_points,res)	end
	end

function love.keypressed( key )
	if key	== '1'		then cp = 1 end
	if key	== '2'		then cp = 2 end
	if key	== '3'		then cp = 3 end
	if key	== '4'		then cp = 4 end
	if key	== '5'		then cp = 5 end
	if key	== '6'		then cp = 6 end
	if key	== '7'		then cp = 7 end
	if key	== '8'		then cp = 8 end
	if key	== '9'		then cp = 9 end
	if key	== '0'		then cp =10 end
	if key	== 'r'		then rotate			= not rotate		end
	if key	== 'p'		then show.path		= not show.path		end
	if key	== 'i'		then show.text		= not show.text		end
	if key	== 'd'		then show.data		= not show.data		end
	if key	== 'c'		then show.circle	= not show.circle	end
	if key	== 'escape'	then love.event.push( 'quit' )			end
	end

function load_screen_settings()
	gr.setCaption( "Tank Movement" )
	width, height	= 800, 600
	gr.setMode( width, height)
	gr.setBackgroundColor( 55, 0, 255, 255 )
	end

function load_defaults()
	tank 		= gr.newImage( 'tanks.png' )
	r			= 0		-- rotation
	cp			= 5		-- active control point (effected by arrow keys)
	-- load all 64 tank images
	tank_size	= 32	-- quad size
	num_tanks	= 8
	quad		= {}
	for y		= 1, num_tanks do
		quad[y] = {}
		for x	= 1, num_tanks do
			quad[y][x] = gr.newQuad( tank_size * (x-1), tank_size * (y-1),
							tank_size, tank_size,
							num_tanks * tank_size, num_tanks * tank_size)
			end
		end
	-- settings for rotating tanks
	radiusX, radiusY	= 275, 275	-- radius for circling tanks
	x,  y, rt, dr		= {}, {}, {}, pi/8
	for i = 1, num_tanks do x[i], y[i] = path(i * dr) end
	rotate				= true
	-- settings for tank following path
	control_points	= { {560, 75},{100,100},{ 60,300},{260,475},{240,150},{400,225},
						{430,510},{560,310},{680,170},{515, 60}}
	res				= 20
	tank_path,curve	= smooth( control_points, res )
	tank_step		= 1		-- first tank to display
	tank_tread		= 1		-- first position for tank tread
	show = { path = false, text = false, data = false, circle = false }
	text = {'Keyboard Controls','<p> to show tank path','(circles are control points)',
		'(red circle is active point)','<arrow keys> move active point',
		'<1,2..0> change active point','<c> displays circling tanks','<r> stops rotation',
		'<d> shows debug info'}
	end

function path( ang )	-- elliptical path for tanks
	return radiusX * cos(ang) + width/2, radiusY * sin(ang) + height/2
	end

function circling_tanks()	-- tanks moving in a circle
	if show.circle then
		for i = 1, num_tanks do
			gr.drawq( tank, quad[i][qdi], x[i], y[i], r+dr*i+pi/2,1,1,16, 16)
			end
		end
	end

function rotating_tank()		-- rotating tank in center of screen
	if show.circle then
		gr.drawq( tank, quad[2][qdi],width/2, height/2,	r+pi,1,1,16, 16)
		end
	end

function path_of_tank()
	if show.path then-- tank path
		gr.setColor(125,125,125,255)
		gr.line( curve )
		-- show control points
		for i = 1, #control_points do
			gr.circle('line',control_points[i][1],control_points[i][2],5,5)
			gr.print(i,control_points[i][1]+5,control_points[i][2]-5)
			end
		-- show which control point is active (moved by arrow keys)
		gr.setColor(255,0,0,255)
		gr.circle('fill',control_points[cp][1],control_points[cp][2],5,5)
		end
	end

function tank_on_path()
	gr.setColor( 100, 100, 100, 255 )
	step = floor( tank_step )		-- only whole number steps
	local ave=math.min(8,#tank_path-step)	-- # points to average movemant over
	if step > #tank_path - ave - 1 then tank_step = 1 step = 1 end
	if #tank_path==step then tank_step = 1 step = 1 end
	tangle = 0
	for i = 0, ave do
		tangle = tangle + orient_tank( tank_path[step+i], tank_path[step+1+i] )
		end
	tangle = tangle / ave
	size = 0.5 + 2 * tank_path[step][2] / height	-- variable tank size
	gr.drawq( tank, quad[4][qdi],tank_path[step][1],tank_path[step][2],tangle,size,size,16, 16)
	end

function text_display()
	gr.print( 'press <i> for coltrols', 10, 580 )
	gr.print( 'fps: '..love.timer.getFPS( ), width-95, height-20 )

	if show.text then
		for i=1,#text do gr.setColor(200,200,50,255) gr.print(text[i],580,i*14) end
		end
	if show.data then
		gr.setColor( 255, 0, 255, 255 )
		gr.print('number of tank steps: '..#tank_path,10,372)
		gr.print('tank location: '..tank_path[step][1]..', '..tank_path[step][2],10,386)
		gr.print('tank size: '..floor(10*size)/10, 10,400)
		for i = 1, #control_points do
			gr.print(i..': '..control_points[i][1]..' - '..control_points[i][2],10,400+i*14)
			end
		end
	end

function orient_tank( p1, p2) 			-- orient tank relative to direction of travel
	return atan2( ( p2[2] - p1[2] ), ( p2[1] - p1[1] ) )
	end

function smooth( points, steps )		-- from gist
	if #points < 3 then	return points end
	local steps		= steps or 5
	local spline	= {}
	local curve		= {}				-- curve added to speed up gr.line()
	local count		= #points - 1
	local p0, p1, p2, p3, x, y
	for i = 1, count do
		if i == 1 then					-- if first point
			p0, p1, p2, p3 = points[i], points[i], points[i + 1], points[i + 2]
			elseif	i == count then		-- if last point
				p0, p1, p2, p3 = points[#points - 2], points[#points - 1], points[#points], points[#points]
			else
			p0, p1, p2, p3 = points[i - 1], points[i], points[i + 1], points[i + 2]
			end
		for t = 0, 1, 1 / steps do
			x = floor(0.5 * ( ( 2 * p1[1] ) + ( p2[1] - p0[1] ) * t + ( 2 * p0[1] - 5 * p1[1] + 4 * p2[1] - p3[1] ) * t * t + ( 3 * p1[1] - p0[1] - 3 * p2[1] + p3[1] ) * t * t * t ))
			y = floor(0.5 * ( ( 2 * p1[2] ) + ( p2[2] - p0[2] ) * t + ( 2 * p0[2] - 5 * p1[2] + 4 * p2[2] - p3[2] ) * t * t + ( 3 * p1[2] - p0[2] - 3 * p2[2] + p3[2] ) * t * t * t ))
			--prevent duplicate entries
			if not(#spline > 0 and spline[#spline][1] == x and spline[#spline][2] == y) then
				table.insert( spline , { x , y } )
				curve[#curve+1]=x
				curve[#curve+1]=y
				end
			end
		end
	return spline,curve
	end
